@@ -7,9 +7,9 @@ http://www.graphlab.com/products/create/tutorials.html.
 
 """
 
-__all__ = ['sframe', 'sarray', 'graph', 'sketch']
+__all__ = ['sframe', 'sarray', 'sgraph', 'sketch']
 
 import sframe
 import sarray
-import graph
+import sgraph
 import sketch
