@@ -10,66 +10,83 @@ import graphlab.connect.main as glconnect
 from graphlab.cython.cy_sarray import UnitySArrayProxy
 from graphlab.cython.context import debug_trace as cython_context
 from graphlab.util import make_internal_url
+import graphlab as gl
 import inspect
 import numpy
 import pandas
 import time
+import array
 
 __all__ = ['SArray']
 
 class SArray(object):
     """
     SArray is an array object scaled to big data.
-    The data in SArray is immutable, homogeneously typed, and is stored column wise.
-    SArray is also used as column in the :py:class:`graphlab.SFrame`.
+    The data in SArray is immutable and homogeneously typed.
+    Each column in an :py:class:`graphlab.SFrame` is an SArray.
 
     SArray can be constructed in various ways:
 
     Construct an SArray from list.
 
     >>> from graphlab import SArray
-    >>> sa = SArray(data=[1,2,3,4,5], int)
+    >>> sa = SArray(data=[1,2,3,4,5], dtype=int)
 
     Construct an SArray from numpy.ndarray.
 
-    >>> sa = gl.SArray(data=numpy.asarray([1,2,3,4,5]), int)
+    >>> sa = SArray(data=numpy.asarray([1,2,3,4,5]), dtype=int)
+    or:
+    >>> sa = SArray(numpy.asarray([1,2,3,4,5]), int)
 
     Construct an SArray from pandas.Series.
 
-    >>> sf = gl.SArray(data=pd.Series([1,2,3,4,5]), int)
+    >>> sa = SArray(data=pd.Series([1,2,3,4,5]), dtype=int)
+    or:
+    >>> sa = SArray(pd.Series([1,2,3,4,5]), int)
 
-    If the type is not specified, automatic inference is attempted via
-    conversion to a Pandas Series
+    If the type is not specified, automatic inference is attempted:
 
-    >>> gl.SArray(data=pd.Series([1,2,3,4,5])).dtype()
+    >>> SArray(data=[1,2,3,4,5]).dtype()
     int
-    >>> gl.SArray(data=pd.Series([1,2,3,4,5.0])).dtype()
+    >>> SArray(data=[1,2,3,4,5.0]).dtype()
     float
 
-    The Datatypes that are supported are, integer, float, string, and
-    list (a list of doubles). For instance:
+    The SArray supports standard datatypes such as: integer, float and string.
+    It also supports three higher level datatypes: float arrays, dict
+    and list (array of arbitrary types).
 
-    Creates an SArray of strings:
+    Create an SArray from a list of strings:
 
     >>> sa = SArray(data=['a','b'])
 
-    Creates an SArray of list:
+    Create an SArray from a list of float arrays;
 
-    >>> sa = SArray(data=[[1.0,1.0], [2.0,2.0]])
+    >>> sa = SArray([[1,2,3], [3,4,5]])
+
+    Create an SArray from a list of lists:
+
+    >>> sa = SArray(data=[['a', 1, {'work': 3}], [2, 2.0]])
+
+    Create an SArray from a list of dictionaries:
+
+    >>> sa = SArray(data=[{'a':1, 'b': 2}, {'b':2, 'c': 1}])
 
     Construct an SArray from local text file. (Only works for local server).
 
-    >>> sf = SArray('/tmp/a_to_z.txt.gz')
+    >>> sa = SArray('/tmp/a_to_z.txt.gz')
 
     Construct an SArray from a text file downloaded from a URL.
 
-    >>> sf = SArray('http://s3-us-west-2.amazonaws.com/testdatasets/a_to_z.txt.gz')
+    >>> sa = SArray('http://s3-us-west-2.amazonaws.com/testdatasets/a_to_z.txt.gz')
 
     Construct an SArray from a text file stored on the server side.
 
-    >>> sf = SArray('remote:///tmp/a_to_z.txt.gz')
+    >>> sa = SArray('remote:///tmp/a_to_z.txt.gz')
 
-    SArrays support a large number of vectorized operations. For instance:
+    **Numeric Operators**
+
+    SArrays support a large number of vectorized operations on numeric types.
+    For instance:
 
     >>> sa = SArray([1,1,1,1,1])
     >>> sb = SArray([2,2,2,2,2])
@@ -94,25 +111,17 @@ class SArray(object):
     Rows: 5
     [0, 1, 1, 1, 0]
 
-    We can also perform logical filtering operations:
-
-    >>> sa = SArray([1,2,3,4,5])
-    >>> sa[(sa >= 2) & (sa <= 4)]
-    dtype: int
-    Rows: 3
-    [2, 3, 4]
-
-    The numeric operators (+,-,*,/) also work on list types:
+    The numeric operators (+,-,*,/) also work on array types:
 
     >>> sa = SArray(data=[[1.0,1.0], [2.0,2.0]])
     >>> sa + 1
     dtype: list
     Rows: 2
-    [[2.0, 2.0], [3.0, 3.0]]
+    [array('f', [2.0, 2.0]), array('f', [3.0, 3.0])]
     >>> sa + sa
     dtype: list
     Rows: 2
-    [[2.0, 2.0], [4.0, 4.0]]
+    [array('f', [2.0, 2.0]), array('f', [4.0, 4.0])]
 
     The addition operator (+) can also be used for string concatenation:
 
@@ -122,6 +131,56 @@ class SArray(object):
     Rows: 2
     ['ax', 'bx']
 
+    All comparison operations and boolean operators are supported and emit
+    binary SArrays.
+
+    >>> sa = SArray([1,2,3,4,5])
+    >>> sa >= 2
+    dtype: int
+    Rows: 3
+    [0, 1, 1, 1, 1]
+    >>> (sa >= 2) & (sa <= 4)
+    dtype: int
+    Rows: 3
+    [0, 1, 1, 1, 0]
+
+    **Logical Filter**
+    
+    An SArray can be filtered using
+
+    >>> array[binary_filter] 
+
+    where array and binary_filter are SArrays of the same length. The result is
+    a new SArray which contains only elements of 'array' where its matching row
+    in the binary_filter is non zero.
+
+    This permits the use of boolean operators that can be used to perform
+    logical filtering operations.  For instance:
+
+    >>> sa = SArray([1,2,3,4,5])
+    >>> sa[(sa >= 2) & (sa <= 4)]
+    dtype: int
+    Rows: 3
+    [2, 3, 4]
+
+    This can also be used more generally to provide filtering capability which
+    is otherwise not expressible with simple boolean functions. For instance:
+
+    >>> sa = SArray([1,2,3,4,5])
+    >>> sa[sa.apply(lambda x: math.log(x) <= 1)]
+    dtype: int
+    Rows: 3
+    [1, 2]
+
+    This is equivalent to 
+
+    >>> sa.filter(lambda x: math.log(x) <= 1)
+    dtype: int
+    Rows: 3
+    [1, 2]
+
+    **Iteration**
+    
     The SArray is also iterable, but not efficiently since this involves a
     streaming transmission of data from the server to the client. This should
     not be used for large data.
@@ -130,6 +189,13 @@ class SArray(object):
     >>> [i + 1 for i in sa]
     [2, 3, 4, 5, 6]
 
+    This can be used to convert an SArray to a list:
+
+    >>> sa = SArray([1,2,3,4,5])
+    >>> l = list(sa)
+    >>> l
+    [1, 2, 3, 4, 5]
+
     Parameters
     ----------
     data : list | numpy.ndarray | pandas.Series | string
@@ -137,8 +203,9 @@ class SArray(object):
         the data in the list is converted and stored in an SArray.
         Alternatively if this is a string, it is inerpreted as a path (or
         url) to a text file.  Each line of the text file is loaded as a
-        separate row. If data is a file name ending with ".sidx", this is
-        loaded as a SArray file.
+        separate row. If data is a directory where an SArray was previously
+        saved, this is loaded as a SArray reading directly out of that
+        directory.
 
     dtype : SArray
         The data type of the SArray. Supported types are: {int,
@@ -162,13 +229,13 @@ class SArray(object):
     However, it is still ok to the remote file path.
 
     >>> graphlab.aws.launch_EC2('m1.large')
-    >>> sf = SArray('~/mydata/foo.csv') # throws exception
-    >>> sf = SArray('remote:///mydata/foo.csv') # works
-    >>> sf = SArray("http://testdatasets.s3-website-us-west-2.amazonaws.com/users.csv.gz") # works
-    >>> sf = SArray("s3://mybucket/foo.csv") # works
+    >>> sa = SArray('~/mydata/foo.csv') # throws exception
+    >>> sa = SArray('remote:///mydata/foo.csv') # works
+    >>> sa = SArray("http://testdatasets.s3-website-us-west-2.amazonaws.com/users.csv.gz") # works
+    >>> sa = SArray("s3://mybucket/foo.csv") # works
     >>> graphlab.aws.teminate_EC2()
 
-    Similar restriction applies to :py:class:`graphlab.Graph` and :py:class:`graphlab.SFrame`.
+    Similar restriction applies to :py:class:`graphlab.SGraph` and :py:class:`graphlab.SFrame`.
     """
 
     def __init__(self, data=[], dtype=None, ignore_cast_failure=False, _proxy=None):
@@ -182,12 +249,13 @@ class SArray(object):
             the data in the list is converted and stored in an SArray.
             Alternatively if this is a string, it is inerpreted as a path (or
             url) to a text file.  Each line of the text file is loaded as a
-            separate row. If data is a file name ending with ".sidx", this is
-            loaded as a SArray file.
+            separate row. If data is a directory where an SArray was previously
+            saved, this is loaded as a SArray reading directly out of that
+            directory.
 
         dtype : SArray
             The data type of the SArray. Supported types are: {int,
-            float, str, list}.  If not specified (None), we attempt to infer
+            float, str, list, dict}.  If not specified (None), we attempt to infer
             it from the input.  If it is a numpy array, or a Pandas
             series, the dtype of the array/series is used. If it is a
             list, it is first converted to a Pandas series, and the
@@ -236,14 +304,10 @@ class SArray(object):
                 with cython_context():
                     self.__proxy__.load_from_iterable(data, dtype, ignore_cast_failure)
             elif (isinstance(data, str)):
-                if data.endswith('.sidx'):
-                    internal_url = make_internal_url(data)
-                    with cython_context():
-                        self.__proxy__.load_from_sarray_index(internal_url)
-                else:
-                    internal_url = make_internal_url(data)
-                    with cython_context():
-                        self.__proxy__.load_from_url(internal_url, dtype)
+                internal_url = make_internal_url(data)
+                with cython_context():
+                    self.__proxy__.load_autodetect(internal_url, dtype)
+
             else:
                 raise TypeError("Unexpected data source. " \
                                 "Possible data source types are: list, " \
@@ -253,25 +317,44 @@ class SArray(object):
         # default is float
         if len(data) == 0:
             return float
+        integral_types = set([int,long,numpy.bool_,numpy.int_,numpy.intc,numpy.intp,
+                              numpy.int8,numpy.int16,numpy.int32,numpy.int64,
+                              numpy.uint8,numpy.uint16,numpy.uint32,numpy.uint64])
+        float_types = set([float,numpy.float_,numpy.float16,numpy.float32,numpy.float64])
+        numeric_types = integral_types.union(float_types)
 
         unique_types = set([type(x) for x in data if x is not None])
         if len(unique_types) == 0:
             return float
-        # if dtype is list, recheck to make sure all values are numeric
         elif len(unique_types) == 1:
-            dtype = unique_types.pop()
-            if dtype != list:
-                return dtype
+            ret = unique_types.pop()
+            # only 1 type. check against basic types, 
+            # and cast it back to int or float
+            if ret in integral_types:
+                return int
+            elif ret in float_types:
+                return float 
+            elif ret != list:
+                # none of the above. its not a list of lists, so its fine.
+                # We can handle the other cases.
+                return ret;
+
+            # if it is a list of lists, we need to look harder at the contents
+            # of each list.
+            # if all contents of the list is numeric, then use vector
+            # otherwise use list
+            value_types = set([])
+            for l in data:
+                value_types = value_types.union(set([type(x) for x in l if l is not None]))
+
+            if value_types.issubset(set([int,long,float])):
+                return array.array
             else:
-                isvalid = [all([isinstance(elem, (int,long,float)) for elem in innerlist]) for innerlist in data]
-                if all(isvalid):
-                    return list
-                else:
-                    raise TypeError("Array type should have all entries as doubles")
+                return list
 
         elif len(unique_types) == 2:
             # we can do both ints, longs, floats as a float
-            if unique_types.issubset(set([int,long,float])):
+            if unique_types.issubset(numeric_types):
                 return float
             else:
                 raise TypeError("Cannot infer Array type. Not all elements of array are the same type.")
@@ -285,14 +368,11 @@ class SArray(object):
         Parameters
         ----------
         filename : string
-            The location to save the SFrame. Either a local path or a remote
-            URL. The filename will get the suffix '.sidx' if it does not already have it.
+            The directory to save the SArray. Either a local path or a remote
+            URL.
         """
-        if not targetfile.endswith('.sidx'):
-            raise ValueError("Save target must end with sidx")
-        else:
-            with cython_context():
-                self.__proxy__.save(targetfile)
+        with cython_context():
+            self.__proxy__.save(make_internal_url(targetfile))
 
     def __repr__(self):
         """
@@ -544,6 +624,21 @@ class SArray(object):
         else:
             raise IndexError("Invalid type to use for indexing")
 
+    def __materialize__(self):
+        """
+        For a SArray that is lazily evaluated, force persist this sarray to disk
+        to enable benchmarking or other usage
+        """
+        with cython_context():
+            self.__proxy__.materialize()
+
+    def __is_materialized__(self):
+        """
+        Returns whether or not the sarray has been materialized. A materialized
+        SArray has all contents saved on disk
+        """
+        return self.__proxy__.is_materialized()
+
     def size(self):
         """
         Returns the size of the sarray.
@@ -583,13 +678,13 @@ class SArray(object):
 
     def vector_slice(self, start, end=None):
         """
-        If this SArray contains vectors, this returns a new SArray containing
-        each individual vector sliced, between start and end, exclusive.
+        If this SArray contains vectors or recursive types, this returns a new SArray
+        containing each individual vector sliced, between start and end, exclusive.
 
-        For instance:
+        For instance, if g is a vector of floats:
 
         >>> g
-        dtype: list
+        dtype: array.array
         Rows: 2
         [[1.0, 2.0, 3.0],
          [2.0, 3.0, 4.0]]
@@ -600,7 +695,7 @@ class SArray(object):
         [1.0, 2.0]
 
         >>> g.vector_slice(0, 2) # extracts the first two elements of each vector
-        dtype: list
+        dtype: array.array
         Rows: 2
         [[1.0, 2.0], [2.0, 3.0]]
 
@@ -609,7 +704,7 @@ class SArray(object):
         For instance:
 
         >>> g
-        dtype: list
+        dtype: array.array
         Rows: 3
         [[1.0], [1.0, 2.0], [1.0, 2.0, 3.0]]
 
@@ -623,6 +718,24 @@ class SArray(object):
         Rows: 3
         [None, [1.0, 2.0], [1.0, 2.0]]
 
+        If g is a vector of mixed types (float, int, str, array, list, etc.):
+        >>> l
+        dtype: list
+        Rows: 2
+        [['a', 1, 1.0],
+         ['b', 2, 2.0]]
+
+        >>> g.vector_slice(0) # extracts the first element of each vector
+        dtype: list
+        Rows: 2
+        [[a'], ['b']]
+
+        >>> g.vector_slice(0, 2) # extracts the first two elements of each vector
+        dtype: list
+        Rows: 2
+        [[a', 1], ['b', 2]]
+
+
         Parameters
         ----------
         start :  The start position of the slice.
@@ -631,7 +744,7 @@ class SArray(object):
         position is NOT included in the slice. Thus a g.vector_slice(1,3) will
         extract entries in position 1 and 2.
         """
-        if self.dtype() != list:
+        if (self.dtype() != array.array) and (self.dtype() != list):
             raise RuntimeError("Only Vector type can be sliced")
         if end == None:
             end = start + 1
@@ -639,6 +752,274 @@ class SArray(object):
         with cython_context():
             return SArray(_proxy=self.__proxy__.vector_slice(start, end))
 
+    def count_words(self, to_lower=True):
+        """
+        Returns a new SArray of dictionary type. Each element contains the word-count
+        for each of the word appeared in corresponding input element. Only SArray of
+        string type is supported.
+
+        For example, if input SArray contains the following element:
+
+            "This is about animal and is also about plants"
+        Then output SArray contains an element that is a dictionary:
+
+            {"this":1, "is":2, "about":2, "and":1, "animal":1, "also":1, plants":1}
+
+
+        Parameters
+        ----------
+        to_lower: bool, optional
+            If True, all word is converted to lower case before counting
+
+        Returns
+        -------
+        out : SArray
+            The SArray of dictionary type, where each element contains the word-count
+            for each of the word appeared in corresponding input element
+
+        """
+        if (self.dtype() != str):
+            raise TypeError("Only SArray of string type is supported for counting bag of words")
+
+        _mt._get_metric_tracker().track('sarray.count_words')
+
+        # construct options, will extend over time
+        options = dict()
+        options["to_lower"] = to_lower == True
+
+        with cython_context():
+            return SArray(_proxy=self.__proxy__.count_bag_of_words(options))
+
+    def dict_trim_by_keys(self, keys, exclude=True):
+        """
+        If SArray dtype is dict, filter out each dict by the given keys.
+        If exclude is True, then all keys that are in the input key list are removed
+        from the dict. exclude is True by default.
+        If exclude is False, then only keys that are in the input key list are
+        retained in the dict.
+
+        For example, if input SArray contains the following element:
+
+        >>> sa = SArray([{'this':1, "is":1, "dog":2}, {"this": 2, "are": 2, "cat": 1}])
+        >>> sa.dict_trim_by_keys(["this", "is", "and", "are"], exclude=True)
+        Out:
+            dict
+            [{"dog":2}, {"cat": 1}]
+
+        >>> sa.dict_trim_by_keys(["dog", "cat", "animal"], exclude=False)
+        Out:
+            dict
+            [{"dog":2}, {"cat": 1}]
+
+        Parameters
+        ----------
+        keys: list
+            A collection of keys to trim down the elements in the SArray
+
+        exclude: bool, optional
+            If True, all keys that are in the input key list are removed, default
+                value is True
+            If False, only keys that are in the input key list are retained.
+
+        Returns
+        -------
+        out : SArray
+            A SArray of dictionary type, with each dict element trimmed according
+            to the input criteria
+
+        """
+        if isinstance(keys, str) or (not hasattr(keys, "__iter__")):
+            keys = [keys]
+
+        _mt._get_metric_tracker().track('sarray.dict_trim_by_keys')
+
+        with cython_context():
+            return SArray(_proxy=self.__proxy__.dict_trim_by_keys(keys, exclude))
+
+    def dict_trim_by_values(self, lower=None, upper=None):
+        """
+        If SArray dtype is dict, filter out each dict by checking the value of
+        the dictionary against the given lower and upper bound. Only the value
+        that falls between the bound are retained (both lower and upper bound
+        are inclusive)
+        This only works if the dictionary value is comparable to the given low/upper
+        values. If not comparable, the values are not trimmed
+
+        For example, if input SArray contains the following element:
+
+        >>> sa = SArray([{'this':1, "is":5, "dog":7}, {"this": 2, "are": 1, "cat": 5}])
+        >>> sa.dict_trim_by_values(2, 5)
+        Out:
+            dict
+            [{"is":5}, {"this": 2, "cat": 5}]
+
+        This is an example only lower bound is given
+        >>> sa = SArray([{'this':1, "is":5, "dog":7}, {"this": 2, "are": 1, "cat": 5}])
+        >>> sa.dict_trim_by_values(2)
+        Out:
+            dict
+            [{"is":5, "dog":7}, {"this": 2, "cat": 5}]
+
+        This is an example only upper bound is given
+        >>> sa = SArray([{'this':1, "is":5, "dog":7}, {"this": 2, "are": 1, "cat": 5}])
+        >>> sa.dict_trim_by_values(upper=5)
+        Out:
+            dict
+            [{'this':1, "is":5}, {"this": 2, "are": 1, "cat": 5}]
+
+        Parameters
+        ----------
+        lower: int, long, float
+            The lowest dictionary value that would be retained in the result. If not
+            given, lower bound is not applied.
+
+        upper: int, long, float
+            The highest dictionary value that would be retained in the result. If not
+            given, upper bound is not applied.
+
+        Returns
+        -------
+        out : SArray
+            A SArray of dictionary type, with each dict element trimmed according to
+            the input criteria
+
+        """
+
+        if None != lower and (not isinstance(lower, (int, float, numpy.integer, float, numpy.float))):
+            raise TypeError("lower bound has to be a numeric value")
+
+        if None != upper and (not isinstance(upper, (int, float, numpy.integer, float, numpy.float))):
+            raise TypeError("upper bound has to be a numeric value")
+
+        _mt._get_metric_tracker().track('sarray.dict_trim_by_values')
+
+        with cython_context():
+            return SArray(_proxy=self.__proxy__.dict_trim_by_values(lower, upper))
+
+    def dict_keys(self):
+        """
+        If SArray dtype is dict, get all keys for each dictionary element and
+        return a new SArray that contains keys from coresponding dictionary
+
+        For example, if input SArray contains the following element:
+
+        >>> sa = SArray([{'this':1, "is":5, "dog":7}, {"this": 2, "are": 1, "cat": 5}])
+        >>> sa.dict_keys()
+        Out:
+            dict
+            [["this", "is", "dog"], ["this", "are", "cat"]]
+
+        Parameters
+        ----------
+
+        Returns
+        -------
+        out : SArray
+            A SArray of list type, with each element is a collection of keys from
+            input SArray element
+
+        """
+        _mt._get_metric_tracker().track('sarray.dict_keys')
+
+        with cython_context():
+            return SArray(_proxy=self.__proxy__.dict_keys())
+
+    def dict_values(self):
+        """
+        If SArray dtype is dict, get all values for each dictionary element and
+        return a new SArray that contains values from coresponding dictionary
+
+        For example, if input SArray contains the following element:
+
+        >>> sa = SArray([{'this':1, "is":5, "dog":7}, {"this": 2, "are": 1, "cat": 5}])
+        >>> sa.dict_values()
+        Out:
+            dict
+            [[1,5,7], [2,1,5]]
+
+        Parameters
+        ----------
+
+        Returns
+        -------
+        out : SArray
+            A SArray of list type, with each element is a collection of values from
+            input SArray element
+
+        """
+        _mt._get_metric_tracker().track('sarray.dict_values')
+
+        with cython_context():
+            return SArray(_proxy=self.__proxy__.dict_values())
+
+    def dict_has_any_keys(self, keys):
+        """
+        If SArray dtype is dict, returns SArray of integer with each element
+        indicates whether or not the original SArray element has any key in the
+        input key list (1 means yes, 0 means no).
+
+        For example, if input SArray contains the following element:
+
+        >>> sa = SArray([{'this':1, "is":5, "dog":7}, {"this": 2, "are": 1, "cat": 5}, {"animal":1}])
+        >>> sa.dict_has_any_keys(["is", "this", "are"])
+        Out:
+            dict
+            [1, 1, 0]
+
+        Parameters
+        ----------
+        keys: list
+            A list of key values to check the dictionary against
+
+        Returns
+        -------
+        out : SArray
+            A SArray of int type, with each element indicates whether or not
+            input SArray element contains any key in the input list
+
+        """
+        if isinstance(keys, str) or (not hasattr(keys, "__iter__")):
+            keys = [keys]
+
+        _mt._get_metric_tracker().track('sarray.dict_has_any_keys')
+
+        with cython_context():
+            return SArray(_proxy=self.__proxy__.dict_has_any_keys(keys))
+
+    def dict_has_all_keys(self, keys):
+        """
+        If SArray dtype is dict, returns SArray of integer with each element
+        indicates whether or not the original SArray element has all keys in the
+        input key list (1 means yes, 0 means no).
+
+        For example, if input SArray contains the following element:
+
+        >>> sa = SArray([{'this':1, "is":5, "dog":7}, {"this": 2, "are": 1, "cat": 5}])
+        >>> sa.dict_has_any_keys(["is", "this"])
+        Out:
+            dict
+            [1, 0]
+
+        Parameters
+        ----------
+        keys: list
+            A list of key values to check the dictionary against
+
+        Returns
+        -------
+        out : SArray
+            A SArray of int type, with each element indicates whether or not
+            input SArray element contains all keys in the input list
+
+        """
+        if isinstance(keys, str) or (not hasattr(keys, "__iter__")):
+            keys = [keys]
+
+        _mt._get_metric_tracker().track('sarray.dict_has_all_keys')
+
+        with cython_context():
+            return SArray(_proxy=self.__proxy__.dict_has_all_keys(keys))
+
     def apply(self, fn, dtype=None, skip_undefined=True, seed=None):
         """
         Returns a new SArray of dtype where each element in this SArray is
@@ -811,7 +1192,7 @@ class SArray(object):
         """
         Returns the sum of all the values in the sarray.  The empty frame
         returns None.
-        If the array contains strings, this will throws an exception.
+        If the array contains strings or dictionary, this will throw an exception.
         If the array contains vectors, and all the vectors are the same length,
         the sum over all the vectors will be returned. Otherwise, if the vectors
         have different lengths, this will throw an exception.
@@ -829,10 +1210,12 @@ class SArray(object):
 
     def mean(self):
         """
-        Returns the mean of the values in the sarray.  The empty frame returns
-        None.  Throws an exception if called on an sarray with non-numeric
+        Returns the mean of all the values in the SArray.  The empty frame returns
+        None.  Throws an exception if called on an SArray with non-numeric
         type.
 
+        If the array contains strings or dictionary, this will throw an exception.
+
         Returns
         -------
         out : float
@@ -847,6 +1230,8 @@ class SArray(object):
         Returns the standard deviation of all the values in the sarray as
         a float.The empty array returns None.
 
+        If the array contains strings or dictionary, this will throw an exception.
+
         Parameters
         ----------
         ddof : int
@@ -871,6 +1256,8 @@ class SArray(object):
         Returns the variance of all the values in the sarray as
         a float. The empty array returns None.
 
+        If the array contains strings or dictionary, this will throw an exception.
+
         Parameters
         ----------
         ddof : int
@@ -937,7 +1324,7 @@ class SArray(object):
             The SArray converted to the dtype
         """
 
-        _mt._get_metric_tracker().track('sarray.astype', {'type':str(dtype)})
+        _mt._get_metric_tracker().track('sarray.astype.%s' % str(dtype.__name__))
 
         with cython_context():
             return SArray(_proxy=self.__proxy__.astype(dtype, undefined_on_failure))
@@ -1092,7 +1479,7 @@ class SArray(object):
         with cython_context():
             return SArray(_proxy = self.__proxy__.topk_index(topk, reverse))
 
-    def sketch_summary(self):
+    def sketch_summary(self, background=False):
         """
         Returns a graphlab.Sketch object which can be further queried for many
         descriptive statistics over this SArray. Many of the statistics are
@@ -1104,7 +1491,58 @@ class SArray(object):
         out : Sketch
             Sketch object that contains descriptive statistics for this SArray.
             Many of the statistics are approximate.
+
+        background : boolean
+          If True, the sketch construction will return immediately and the
+          sketch will be constructed in the background. While this is going on,
+          the sketch can be queried incrementally, but at a performance penalty.
+          Defaults to False.
         """
         from graphlab.data_structures.sketch import Sketch
         _mt._get_metric_tracker().track('sarray.sketch_summary')
-        return Sketch(self)
+        return Sketch(self, background)
+
+    def append(self, other):
+        """
+        Append the second SArray to current SArray. Returns a new SArray contains rows from both SArrays.
+        Both SArrays have to have the same type.
+
+        Parameters
+        ----------
+        other : SArray
+            Another SArray whose rows are appended to current SArray
+
+        Returns
+        -------
+        out_sf : SArray
+            A new SArray that contains rows from both SArrays, with second SArray's rows after
+            first SArray's rows
+        """
+        _mt._get_metric_tracker().track('sarray.append')
+        if type(other) is not SArray:
+            raise RuntimeError("SArray append can only work with SArray")
+
+        if self.dtype() != other.dtype():
+            raise RuntimeError("Data types in both SArrays have to be the same")
+
+        with cython_context():
+            return SArray(_proxy = self.__proxy__.append(other.__proxy__))
+
+    def unique(self):
+        """
+        Return an SArray containing only the unique values of the given SArray.
+        Will not necessarily preserve the order of the given SArray in the new
+        SArray.
+
+        Returns
+        -------
+        out : SArray
+            A new SArray that contains the unique values of the given SArray.
+        """
+        _mt._get_metric_tracker().track('sarray.unique')
+        tmp_sf = gl.SFrame()
+        tmp_sf.add_column(self, 'X1')
+
+        res = tmp_sf.groupby('X1',{}) 
+
+        return SArray(_proxy=res['X1'].__proxy__)
