@@ -24,10 +24,14 @@ except:
 
 class KmeansModel(Model):
     """
-    A Model object containing a cluster id for each vertex, and the centers
+    A model object containing a cluster id for each vertex, and the centers
     of the clusters.
+
+    An instance of this model can be created using :func:`graphlab.kmeans.create`.
+    Do NOT construct the model directly.
     """
     def __init__(self, input_params, model):
+        '''__init__(self)'''
         self.__proxy__ = model
         self._input = input_params
         self._fields = ['clusterid', 'runtime', 'num_iterations', 'cluster_info']
