@@ -1,5 +1,5 @@
 
-from graphlab.data_structures.graph import Graph
+from graphlab.data_structures.sgraph import SGraph
 import graphlab.toolkits.main as main
 from graphlab.toolkits.model import Model
 from collections import deque
@@ -7,26 +7,67 @@ from collections import deque
 
 class ShortestPathModel(Model):
     """
-    A Model object containing the distance for each vertex in the graph to the
+    A model object containing the distance for each vertex in the graph to the
     source vertex.
+
+    An instance of this model can be created using :func:`graphlab.shortest_path.create`.
+    Do NOT construct the model directly.
     """
     def __init__(self, input_params, model):
+        '''__init__(self)'''
         self.__proxy__ = model
         self._input = input_params
         self._fields = ['runtime', 'distance', 'graph']
 
     def list_fields(self):
-        """Return the list of fields in the model."""
+        """
+        List of fields stored in the model. Each of these fields can be queried
+        using the ``get`` function.
+
+        Returns
+        -------
+        out : list
+            A list of fields that can be queried using the ``get`` method.
+        """
 
         return self._fields
 
     def get(self, field):
-        """Return the value for the queried field."""
+        """
+        Get the value of a given field. The list of all queryable fields is
+        detailed below, and can be obtained programmatically using the
+        :func:`~graphlab.graph_analytics.ShortestPathModel.list_fields`
+        method.
+
+        Each of these fields can be queried in one of two ways:
+
+        >>> out = m['field']
+        >>> out = m.get('field')  # equivalent to previous line
+
+        +-----------+----------------------------------------------------------+
+        |   Field   | Description                                              |
+        +===========+==========================================================+
+        | distance  | SFrame with each vertex's distance to the queried vertex |
+        +-----------+----------------------------------------------------------+
+        | graph     | Input graph with the distance as a vertex property       |
+        +-----------+----------------------------------------------------------+
+
+        Parameters
+        ----------
+        field : string
+            Name of the field to be retrieved.
+
+        Returns
+        -------
+        out : [various]
+            The current value of the requested field.
+
+       """
 
         if (field == 'distance'):
-            return Graph(_proxy=self.__proxy__.get('__graph__')).select_fields(['distance']).get_vertices()
+            return SGraph(_proxy=self.__proxy__.get('__graph__')).select_fields(['distance']).get_vertices()
         elif (field == 'graph'):
-            return Graph(_proxy=self.__proxy__.get('__graph__'))
+            return SGraph(_proxy=self.__proxy__.get('__graph__'))
         else:
             return self.__proxy__.get(field)
 
@@ -108,7 +149,7 @@ class ShortestPathModel(Model):
         path = list(path)
 
         if show is True and len(path) > 1:
-            sub_g = Graph()
+            sub_g = SGraph()
             for i, j in zip(path, path[1:]):
                 sub_g = sub_g.add_edges(g.get_edges(src_ids=i, dst_ids=j),
                                         src_field='__src_id',
@@ -128,17 +169,17 @@ class ShortestPathModel(Model):
         return lambda m: ShortestPathModel(self._input, m)
 
 
-def create(graph, source_vid, weight_field="", verbose=False, plot=False):
+def create(graph, source_vid, weight_field="", verbose=True, plot=False):
     """
     Compute the single source shortest path distance from the source vertex to
-    all vertices in the graph. Note that because Graph is directed, shortest
+    all vertices in the graph. Note that because SGraph is directed, shortest
     paths are also directed. To find undirected shortes paths add edges to the
-    Graph in both directions. Return a model object with distance each of vertex in the
+    SGraph in both directions. Return a model object with distance each of vertex in the
     graph.
 
     Parameters
     ----------
-    graph : Graph
+    graph : SGraph
         The graph on which to compute shortest paths.
 
     source_vid : vertex ID
@@ -162,7 +203,7 @@ def create(graph, source_vid, weight_field="", verbose=False, plot=False):
 
     Examples
     --------
-    If given an :class:`~graphlab.Graph` ``g``, we can create
+    If given an :class:`~graphlab.SGraph` ``g``, we can create
     a :class:`~graphlab.shortest_path.ShortestPathModel` as follows:
 
     >>> sp = shortest_path.create(g, source_vid=123)
@@ -170,24 +211,19 @@ def create(graph, source_vid, weight_field="", verbose=False, plot=False):
     We can obtain the shortest path from the source vertex to each vertex in
     the graph ``g`` as follows:
 
-    >>> sp_sframe = sp.get('distance')   # SFrame
+    >>> sp_sframe = sp['distance']   # SFrame
 
     We can obtain an auxiliary graph with additional information corresponding
     to the shortest path from the source vertex to each vertex in the graph
     ``g`` as follows:
 
-    >>> sp_graph = sp.get('graph')       # Graphlab graph
+    >>> sp_graph = sp.get['graph']       # SGraph
     """
-    if not isinstance(graph, Graph):
-        raise TypeError('graph input must be a Graph object.')
+    if not isinstance(graph, SGraph):
+        raise TypeError('graph input must be a SGraph object.')
 
     if plot is True:
         print "The plot functionality for shortest path is not yet implemented."
-        plot = False
-
-    if verbose is True:
-        print "Starting shortest path computation."
-        verbose = 0
 
     opts = {'source_vid': source_vid, 'edge_attr': weight_field,
             'graph': graph.__proxy__}
