@@ -7,47 +7,50 @@ import graphlab as _graphlab
 from graphlab.toolkits.recommender.recommender import RecommenderModel
 from graphlab.data_structures.sframe import SFrame as _SFrame
 from pandas import DataFrame as _DataFrame
+import logging
 
 
-def create(dataset, user, item, target,
-           holdout_probability=0.0,
-           verbose=True, plot=False, random_seed=0, **kwargs):
+def create(dataset,
+           user_column='user_id', item_column='item_id',
+           target_column='target',
+           user_data=None, item_data=None,
+           verbose=True, plot=False,
+           **method_options):
     """
     Create an :class:`~graphlab.recommender.item_means.ItemMeansModel` that uses
     each item's mean rating. These mean ratings are used when scoring (user,
     item) pairs and making recommendations, regardless of user.
 
+    NOTE: This will be *deprecated* soon. Please use
+    graphlab.recommender.create instead.
+
     Parameters
     ----------
-    dataset : pandas.DataFrame/SFrame
+    dataset : SFrame
         The dataset to use for training the model.
 
-    user : string
+    user_column : string
         The column name of the dataset that corresponds to user id.
 
-    item : string
+    item_column : string
         The column name of the dataset that corresponds to item id.
 
-    target : string
-        The column name of the dataset that corresponds to the target of
-        interest, e.g. the rating a user gave a particular item.
+    target_column : string
+        The model will be trained to predict this column of the data set.
 
-    holdout_probability : float, optional
-        Proportion of the dataset held out of model training. The held-out
-        subset is used to estimate the model's error rate when making
-        predictions with new data; the prediction is available in the model's
-        training_stats field.
+    user_data : SFrame
+        Side information for the users.  This SFrame must have a column named
+        with the  user column name given above. It provides any amount of
+        additional user-specific information.
+
+    item_data: SFrame
+        Side information for the items. This SFrame must have a column named
+        with the item column name given above. It provides any amount of
+        additional item-specific information.
 
     verbose : bool, optional
         If True, print progress updates.
 
-    plot : bool, optional
-        If True, display the progress plot.
-
-    random_seed : integer, optional
-        The random seed used to choose the training and validation
-        split if holdout_probability is nonzero.
-
     Returns
     -------
     out : ItemMeansModel
@@ -76,6 +79,8 @@ def create(dataset, user, item, target,
 
     _mt._get_metric_tracker().track('toolkit.recsys.item_means.create')
 
+    logging.warning("This method will be deprecated soon. Please switch to recommender.create")
+
     if not isinstance(dataset, (_DataFrame, _SFrame)):
         raise TypeError('dataset input must be a pandas.DataFrame or SFrame')
 
@@ -86,29 +91,32 @@ def create(dataset, user, item, target,
         print "The plot functionality for item means is not yet implemented."
         plot = False
 
-    opts = {'model_name': 'item_counts'}
-    response = _graphlab.toolkits.main.run("recsys_init", opts)
-    model_proxy = response['model']
+    m = _graphlab.recommender.create(dataset,
+            user_column=user_column, item_column=item_column,
+            target_column=target_column,
+            user_data=user_data, item_data=item_data,
+            verbose=verbose,
+            method='item_means',
+            **method_options)
 
-    opts = {'user_column': user,
-            'item_column': item,
-            'target_column': target,
-            'training_data': dataset,
-            'random_seed' : random_seed,
-            'model': model_proxy,
-            'holdout_probability': holdout_probability}
-    opts.update(kwargs)
-    response = _graphlab.toolkits.main.run("recsys_train", opts, verbose, plot)
-
-    return ItemMeansModel(model_proxy)
+    return m
 
 
 class ItemMeansModel(RecommenderModel):
     """
     Use each item's mean rating to predict the target variable, regardless of the user.
+
+    An instance of this model can be created using
+    :func:`create(..., method='item_means') <graphlab.recommender.create>`.
+    Do NOT construct the model directly.
+
+    NOTE: This will be deprecated soon. Please use
+    :func:`create(..., method='item_means') <graphlab.recommender.create>`
+    instead.
     """
 
     def __init__(self, model_proxy):
+        '''__init___(self)'''
         self.__proxy__ = model_proxy
 
     def _get_wrapper(self):
