@@ -7,12 +7,16 @@ import graphlab as _graphlab
 from graphlab.toolkits.recommender.recommender import RecommenderModel
 from graphlab.data_structures.sframe import SFrame as _SFrame
 from pandas import DataFrame as _DataFrame
+import warnings
+import logging
 
-
-def create(dataset, user, item, target=None, similarity_type='jaccard',
-           only_top_k=0, holdout_probability=0.0, threshold=0.0,
+def create(dataset,
+           user_column='user_id', item_column='item_id', target_column=None,
+           user_data=None, item_data=None,
+           similarity_type='jaccard',
+           only_top_k=0, threshold=0.0,
            threshold_degree=None, threshold_window=None,
-           verbose=True, plot=False, random_seed=0, **kwargs):
+           verbose=True, plot=False, **method_options):
     """
     Create an :class:`~graphlab.recommender.item_similarity.ItemSimilarityModel`.
     This scores an item according to its similarity to other items observed for the
@@ -20,20 +24,34 @@ def create(dataset, user, item, target=None, similarity_type='jaccard',
     for each item, e.g. the rating given to both items by users who have rated
     both items.
 
+    NOTE: This will be deprecated soon. Please use
+    graphlab.recommender.create instead.
+
+
     Parameters
     ----------
-    dataset : pandas.DataFrame/SFrame
+    dataset : SFrame
         The dataset to use for training the model.
 
-    user : string
+    user_column : string
         The column name of the dataset that corresponds to user id.
 
-    item : string
+    item_column : string
         The column name of the dataset that corresponds to item id.
 
-    target : string, optional
-        The column name of the dataset that corresponds to the target of
-        interest, e.g. the rating a user gave a particular item.
+    target_column : string
+        If a score is present in the information -- such as a rating -- then
+        this specifies the name of the column in the data to use as the target variable.
+
+    user_data : SFrame
+        Side information for the users.  This SFrame must have a column named
+        with the  user column name given above. It provides any amount of
+        additional user-specific information.
+
+    item_data: SFrame
+        Side information for the items. This SFrame must have a column named
+        with the item column name given above. It provides any amount of
+        additional item-specific information.
 
     similarity_type : {'jaccard', 'cosine'}, optional
         Measure of similarity between two items. The allowed functions are:
@@ -48,22 +66,12 @@ def create(dataset, user, item, target=None, similarity_type='jaccard',
         scores. When many unique items exist, using k>0 can help reduce
         the amount of memory required.
 
-    holdout_probability : float, optional
-        Proportion of the dataset held out of model training. The held-out
-        subset is used to estimate the model's error rate when making
-        predictions with new data; the prediction is available in the model's
-        training_stats field.
-
     verbose : bool, optional
         If True, print progress updates.
 
     plot : bool, optional
         If True, display the progress plot.
 
-    random_seed : integer, optional
-        The random seed used to choose the training and validation
-        split if holdout_probability is nonzero.
-
     threshold : float, optional
         Any items that have similarity below this threshold are ignored during
         prediction.
@@ -114,6 +122,8 @@ def create(dataset, user, item, target=None, similarity_type='jaccard',
 
     _mt._get_metric_tracker().track('toolkit.recsys.item_similarity.create')
 
+    logging.warning("item_similarity.create will be deprecated soon. Please switch to recommender.create")
+
     if not isinstance(dataset, (_DataFrame, _SFrame)):
         raise TypeError('dataset input must be a pandas.DataFrame or SFrame')
 
@@ -124,35 +134,26 @@ def create(dataset, user, item, target=None, similarity_type='jaccard',
         print "The plot functionality for item similarity is not yet implemented."
         plot = False
 
-    if target is None:
-        target = ''
-
-    opts = {'model_name': 'itemcf',
-            'similarity_type': similarity_type}
-    response = _graphlab.toolkits.main.run("recsys_init", opts)
-    model_proxy = response['model']
-
-    opts = {'user_column': user,
-            'item_column': item,
-            'target_column': target,
-            'training_data': dataset,
-            'model': model_proxy,
-            'random_seed' : random_seed,
-            'only_top_k' : only_top_k,
-            'threshold': threshold,
-            'holdout_probability': holdout_probability}
-    opts.update(kwargs)
+    method_options = {'similarity_type': similarity_type,
+                      'only_top_k': only_top_k,
+                      'threshold': threshold}
 
     # Add threshold options if provided by the user.
     if threshold_degree is not None:
-        opts.update({'threshold_lower_degree': threshold_degree})
+        method_options.update({'threshold_lower_degree': threshold_degree})
 
     if threshold_window is not None:
-        opts.update({'threshold_window_degree': threshold_window})
+        method_options.update({'threshold_window_degree': threshold_window})
 
-    response = _graphlab.toolkits.main.run("recsys_train", opts, verbose, plot)
 
-    return ItemSimilarityModel(model_proxy)
+    m = _graphlab.recommender.create(dataset,
+            user_column=user_column, item_column=item_column, target_column=target_column,
+            user_data=user_data, item_data=item_data,
+            verbose=verbose,
+            method='item_similarity',
+            **method_options)
+
+    return m
 
 
 class ItemSimilarityModel(RecommenderModel):
@@ -161,12 +162,22 @@ class ItemSimilarityModel(RecommenderModel):
     observed for the user in question. The similarity can be computed based on
     the observed users for each item, e.g. the rating given to both items by
     users who have rated both items.
+
+    An instance of this model can be created using
+    :func:`create(..., method='item_similarity') <graphlab.recommender.create>`.
+    Do NOT construct the model directly.
+
+    NOTE: This will be deprecated soon. Please use
+    :func:`create(..., method='item_similarity') <graphlab.recommender.create>`
+    instead.
     """
 
     def __init__(self, model_proxy):
+        '''__init__(self)'''
         self.__proxy__ = model_proxy
 
     def _get_wrapper(self):
         def model_wrapper(model_proxy):
             return ItemSimilarityModel(model_proxy)
         return model_wrapper
+
