@@ -8,9 +8,12 @@ import graphlab as _graphlab
 from graphlab.toolkits.recommender.recommender import RecommenderModel
 from graphlab.data_structures.sframe import SFrame as _SFrame
 from pandas import DataFrame as _DataFrame
+import logging
 
-def create(dataset, user, item, target, reg=1e-5,
-           holdout_probability=0.0, sgd_step_size=0.1, max_iterations=1000,
+def create(dataset,
+           user_column='user_id', item_column='item_id',
+           user_data=None, item_data=None,
+           reg=1e-5, sgd_step_size=0, max_iterations=50,
            verbose=True, plot=False,
            random_seed = 0,
            **kwargs):
@@ -43,32 +46,32 @@ def create(dataset, user, item, target, reg=1e-5,
     categorical variables alongside users and items; if they are typed as
     floating point numbers, then they are treated as continuous values.
 
+    NOTE: This will be deprecated soon. Please use
+    graphlab.recommender.create instead.
+
+
     Parameters
     ----------
     dataset : SFrame/DataFrame
         The dataset to use for training the model.
 
-    user : string
+    user_column : string
         The column name of the dataset that corresponds to user id.
 
-    item : string
+    item_column : string
         The column name of the dataset that corresponds to item id.
 
-    target : string
+    target_column : string
         The model will be trained to predict the values in this column.
 
     reg : float, optional
         Regularization weight.
 
-    holdout_probability : float, optional
-        Proportion of the dataset held out of model training. The held-out
-        subset is used to estimate the model's error rate when making
-        predictions with new data; the prediction is available in the model's
-        training_stats field.
-
     sgd_step_size : float, optional
         Step size for stochastic gradient descent. Smaller values may
-        add accuracy but cause the model to converge more slowly.
+        add accuracy but cause the model to converge more slowly.  If
+        zero (default), the step size is chosen by trying several options
+        on a small subset of the data.
 
     max_iterations : int, optional
         Maximum number of iterations through the data.
@@ -80,9 +83,8 @@ def create(dataset, user, item, target, reg=1e-5,
         If True, display the progress plot.
 
     random_seed : integer, optional
-        The random seed used to choose the training and validation
-        split if holdout_probability is nonzero.  It is also used to
-        choose a random starting point with which to initialize the model.
+        The random seed used to choose a random starting point with
+        which to initialize the model.
 
     Returns
     -------
@@ -116,6 +118,7 @@ def create(dataset, user, item, target, reg=1e-5,
     """
 
     _mt._get_metric_tracker().track('toolkit.recsys.linear_regression.create')
+    logging.warning("This method will be deprecated soon. Please switch to recommender.create")
 
     if not isinstance(dataset, (_DataFrame, _SFrame)):
         raise TypeError('dataset input must be a pandas.DataFrame or SFrame')
@@ -127,24 +130,19 @@ def create(dataset, user, item, target, reg=1e-5,
         print "The plot functionality for linear_regression is not yet implemented."
         plot = False
 
-    opts = {'model_name': 'linear_regression'}
-    response = _graphlab.toolkits.main.run("recsys_init", opts)
-    model_proxy = response['model']
+    method_options = {'regularization': reg,
+                      'sgd_step_size': sgd_step_size,
+                      'max_iterations':max_iterations}
+    method_options.update(kwargs)
 
-    opts = {'user_column': user,
-            'item_column': item,
-            'target_column': target,
-            'training_data': dataset,
-            'linear_regularization' : reg,
-            'random_seed' : random_seed,
-            'max_iterations' : max_iterations,
-            'model': model_proxy,
-            'holdout_probability': holdout_probability}
+    m = _graphlab.recommender.create(dataset,
+            user_column=user_column, item_column=item_column,
+            user_data=user_data, item_data=item_data,
+            verbose=verbose,
+            method='linear_model',
+            **method_options)
 
-    opts.update(kwargs)
-    response = _graphlab.toolkits.main.run("recsys_train", opts, verbose, plot)
-
-    return LinearRegressionModel(model_proxy)
+    return m
 
 
 class LinearRegressionModel(RecommenderModel):
@@ -156,9 +154,18 @@ class LinearRegressionModel(RecommenderModel):
 
     See Wikipedia for a description of linear regression:
     http://en.wikipedia.org/wiki/Linear_regression
+
+    An instance of this model can be created using
+    :func:`create(..., method='linear_model') <graphlab.recommender.create>`.
+    Do NOT construct the model directly.
+
+    NOTE: This will be deprecated soon. Please use
+    :func:`create(..., method='linear_model') <graphlab.recommender.create>`
+    instead.
     """
 
     def __init__(self, model_proxy):
+        '''__init__(self)'''
         self.__proxy__ = model_proxy
 
     def _get_wrapper(self):
