@@ -9,10 +9,14 @@ from graphlab.toolkits.recommender.recommender import RecommenderModel
 from graphlab.data_structures.sframe import SFrame as _SFrame
 import time
 from pandas import DataFrame as _DataFrame
+import logging
 
-
-def create(dataset, user, item, target, D=10, reg=0.01, nmf=False,
-           use_bias=True, holdout_probability=0.0, niter=10,
+def create(dataset,
+           user_column='user_id', item_column='item_id',
+           target_column='target',
+           user_data=None, item_data=None,
+           n_factors=10, regularization=0.01, nmf=False,
+           max_iterations=50,
            verbose=True, plot=False, random_seed=None, **kwargs):
     """
     Create a :class:`~graphlab.recommender.matrix_factorization.MatrixFactorizationModel`.
@@ -21,6 +25,8 @@ def create(dataset, user, item, target, D=10, reg=0.01, nmf=False,
     This is an implementation of basic Alternating Least Squares with
     user bias, item bias and a global bias term.
 
+    NOTE: This will be *deprecated* soon. Please use graphlab.recommender.create instead.
+
     References
     ----------
     See this paper for details about the model: Koren, Yehuda, Robert Bell and
@@ -29,40 +35,41 @@ def create(dataset, user, item, target, D=10, reg=0.01, nmf=False,
 
     Parameters
     ----------
-    dataset : pandas.DataFrame/SFrame
+    dataset : SFrame
         The dataset to use for training the model.
 
-    user : string
+    user_column : string
         The column name of the dataset that corresponds to user id.
 
-    item : string
+    item_column : string
         The column name of the dataset that corresponds to item id.
 
-    target : string
+    target_column : string
         The model will be trained to predict this column of the data set.
 
-    D : int, optional
+    user_data : SFrame
+        Side information for the users.  This SFrame must have a column named
+        with the  user column name given above. It provides any amount of
+        additional user-specific information.
+
+    item_data: SFrame
+        Side information for the items. This SFrame must have a column named
+        with the item column name given above. It provides any amount of
+        additional item-specific information.
+
+    n_factors : int, optional
         Number of latent factors to use for describing each item (default is
         10).
 
-    reg : float, optional
-        Regularization weight (default is 0.01)
+    regularization : float, optional
+
+    Regularization weight (default is 0.01)
 
     nmf : boolean, optional
         Non-negative matrix factorization flag. If True, the model is constrained
         to use only non-negative matrices (default is False).
 
-    use_bias : boolean, optional
-        Bias term flag. If True (default), the model includes user and item bias
-        terms.
-
-    holdout_probability : float, optional
-        Proportion of the dataset held out of model training. The held-out
-        subset is used to estimate the model's error rate when making
-        predictions with new data; the prediction is available in the model's
-        training_stats field.
-
-    niter : int, optional
+    max_iterations : int, optional
         Number of optimization iterations to run.
 
     verbose : bool, optional
@@ -72,8 +79,7 @@ def create(dataset, user, item, target, D=10, reg=0.01, nmf=False,
         If True, display the progress plot.
 
     random_seed : integer, optional
-        The random seed used to choose the training and validation
-        split if holdout_probability is nonzero.
+        The random seed used to choose the starting point of the optimization.
 
     Returns
     -------
@@ -102,6 +108,8 @@ def create(dataset, user, item, target, D=10, reg=0.01, nmf=False,
 
     _mt._get_metric_tracker().track('toolkit.recsys.matrix_factorization.create')
 
+    logging.warning("matrix_factorization.create will be deprecated soon. Please switch to recommender.create")
+
     if not isinstance(dataset, (_DataFrame, _SFrame)):
         raise TypeError('dataset input must be a pandas.DataFrame or SFrame')
 
@@ -111,27 +119,23 @@ def create(dataset, user, item, target, D=10, reg=0.01, nmf=False,
     if not random_seed:
         random_seed = int(time.time())
 
-    opts = {'model_name': 'matrix_factorization',
-            'D': D,
-            'regularizer': reg,
-            'nmf': nmf,
-            'use_bias': use_bias}
+    method_options = {'model_name': 'matrix_factorization',
+                      'n_factors': n_factors,
+                      'regularization': regularization,
+                      'nmf': nmf,
+                      'max_iterations': max_iterations}
+    method_options.update(kwargs)
 
-    response = _graphlab.toolkits.main.run("recsys_init", opts)
-    model_proxy = response['model']
+    m = _graphlab.recommender.create(dataset,
+                                     user_column=user_column,
+                                     item_column=item_column,
+                                     target_column=target_column,
+                                     user_data=user_data, item_data=item_data,
+                                     verbose=verbose,
+                                     method='matrix_factorization',
+                                     **method_options)
 
-    opts = {'user_column': user,
-            'item_column': item,
-            'target_column': target,
-            'training_data': dataset,
-            'model': model_proxy,
-            'random_seed' : random_seed,
-            'holdout_probability': holdout_probability,
-            'niter': niter}
-    opts.update(kwargs)
-    response = _graphlab.toolkits.main.run("recsys_train", opts, verbose, plot)
-
-    return MatrixFactorizationModel(model_proxy)
+    return m
 
 
 class MatrixFactorizationModel(RecommenderModel):
@@ -140,9 +144,19 @@ class MatrixFactorizationModel(RecommenderModel):
     In other words, the method will compose a set of user-item ratings (when represented as a matrix) into a
     number of latent factors for each item and corresponding weights for each
     user.
+
+    An instance of this model can be created using
+    :func:`create(..., method='matrix_factorization') <graphlab.recommender.create>`.
+    Do NOT construct the model directly.
+
+    NOTE: This will be deprecated soon. Please use
+    :func:`create(..., method='matrix_factorization') <graphlab.recommender.create>`
+    instead.
+
     """
 
     def __init__(self, model_proxy):
+        '''__init__(self)'''
         self.__proxy__ = model_proxy
 
     def _get_wrapper(self):
