@@ -6,44 +6,48 @@ import graphlab as _graphlab
 from graphlab.toolkits.recommender.recommender import RecommenderModel
 from graphlab.data_structures.sframe import SFrame as _SFrame
 from pandas import DataFrame as _DataFrame
+import logging
 
 
-def create(dataset, user, item,
-           holdout_probability=0.0,
-           verbose=True, plot=False, random_seed=0, **kwargs):
+def create(dataset,
+           user_column='user_id', item_column='item_id',
+           user_data=None, item_data=None,
+           verbose=True, plot=False, random_seed=0,
+           **method_options):
     """
     Create a :class:`~graphlab.recommender.popularity.PopularityModel` that
     ranks items based on their overall popularity. Items are scored by the
     number of times each is seen in the training set. A trained model can be
     used to score (user, item) pairs and make recommendations.
 
+    NOTE: This will be *deprecated* soon. Please use
+    graphlab.recommender.create instead.
+
+
     Parameters
     ----------
-    dataset : SFrame/pandas.DataFrame
+    dataset : SFrame
         The dataset to use for training the model.
 
-    user : string
+    user_column : string
         The column name of the dataset that corresponds to user id.
 
-    item : string
+    item_column : string
         The column name of the dataset that corresponds to item id.
 
-    holdout_probability : float, optional
-        Proportion of the dataset held out of model training. The held-out
-        subset is used to estimate the model's error rate when making
-        predictions with new data; the prediction is available in the model's
-        training_stats field.
+    user_data : SFrame
+        Side information for the users.  This SFrame must have a column named
+        with the  user column name given above. It provides any amount of
+        additional user-specific information.
+
+    item_data: SFrame
+        Side information for the items. This SFrame must have a column named
+        with the item column name given above. It provides any amount of
+        additional item-specific information.
 
     verbose : bool, optional
         If True, print progress updates.
 
-    plot : bool, optional
-        If True, display the progress plot.
-
-    random_seed : integer, optional
-        The random seed used to choose the training and validation
-        split if holdout_probability is nonzero.
-
     Returns
     -------
     out : PopularityModel
@@ -71,6 +75,7 @@ def create(dataset, user, item,
     """
 
     _mt._get_metric_tracker().track('toolkit.recsys.popularity.create')
+    logging.warning("This method will be deprecated soon. Please switch to recommender.create")
 
     if not isinstance(dataset, (_DataFrame, _SFrame)):
         raise TypeError('dataset input must be a pandas.DataFrame or SFrame')
@@ -82,29 +87,32 @@ def create(dataset, user, item,
         print "The plot functionality for popularity is not yet implemented."
         plot = False
 
-    opts = {'model_name': 'item_counts'}
-    response = _graphlab.toolkits.main.run("recsys_init", opts)
-    model_proxy = response['model']
+    m = _graphlab.recommender.create(dataset,
+            user_column=user_column, item_column=item_column,
+            user_data=user_data, item_data=item_data,
+            verbose=verbose,
+            method='popularity',
+            **method_options)
 
-    opts = {'user_column': user,
-            'item_column': item,
-            'training_data': dataset,
-            'model': model_proxy,
-            'random_seed' : random_seed,
-            'holdout_probability': holdout_probability}
-    opts.update(kwargs)
-    response = _graphlab.toolkits.main.run("recsys_train", opts, verbose, plot)
-
-    return PopularityModel(model_proxy)
+    return m
 
 
 class PopularityModel(RecommenderModel):
     """
-    Item popularity model for the GraphLab recommender system. Each user's
+    A model that ranks an item according to its popularity. Each user's
     recommendations are the overall most popular items in the data.
+
+    An instance of this model can be created using
+    :func:`create(..., method='popularity') <graphlab.recommender.create>`.
+    Do NOT construct the model directly.
+
+    NOTE: This will be deprecated soon. Please use
+    :func:`create(..., method='popularity') <graphlab.recommender.create>`
+    instead.
     """
 
     def __init__(self, model_proxy):
+        '''__init__(self)'''
         self.__proxy__ = model_proxy
 
     def _get_wrapper(self):
