@@ -7,7 +7,9 @@ from graphlab.data_structures.sarray import SArray as _SArray
 from graphlab.data_structures.sframe import SFrame as _SFrame
 import pandas as _pd
 import json as _json
-
+import array
+import numpy as _np
+import logging as _logging
 
 class RecommenderModel(Model):
     """
@@ -27,7 +29,7 @@ class RecommenderModel(Model):
             A dictionary that maps model options to the values used during
             training.
         """
-        return self._get_default_options().keys()
+        return self._get_current_options().keys()
 
     def get(self, field):
         """
@@ -48,10 +50,20 @@ class RecommenderModel(Model):
         return response['value']
 
     def summary(self):
-        """Returns a summary of training statistics."""
-        _mt._get_metric_tracker().track('toolkit.recsys.summary')
+        """Returns a summary of statistics about the model, the data
+        used to train the model, and the training performance.
 
-        return self.training_stats()
+        Returns
+        -------
+        out : dict
+            A dictionary containing model parameters, summary statistics of the
+            data set, and summary statistics about training.
+
+        """
+        _mt._get_metric_tracker().track('toolkit.recsys.summary')
+        opts = {'model': self.__proxy__}
+        response = _graphlab.toolkits.main.run('recsys_summary', opts)
+        return response
 
     def __str__(self):
         """
@@ -76,7 +88,7 @@ class RecommenderModel(Model):
             A description of the model.
         """
 
-        stats = self.training_stats()
+        stats = self.summary()
         options = self._get_current_options()
 
         # Print model type and data schema
@@ -97,45 +109,52 @@ class RecommenderModel(Model):
                     del options[k]
 
             # Statistics about the training and validation set
+            ret += "\nStatistics\n"
             ks = ["num_obs_train",
-                  "num_unique_users_train",
-                  "num_unique_items_train",
-                  "num_obs_validate",
-                  "num_unique_users_validate",
-                  "num_unique_items_validate"]
+                  "num_unique_users",
+                  "num_unique_items"]
             if all([k in stats.keys() for k in ks]):
-                ret += "\nStatistics\n"
                 ret += "  %-17s %11d obs %11d users %11d items\n" % \
                     ("Training set:", stats[ks[0]], stats[ks[1]], stats[ks[2]])
 
-                # Only show validation description if there are observations
-                # present
-                if stats["num_obs_validate"] > 0:
-                    ret += "  %-17s %11d obs %11d users %11d items\n" % \
-                        ("Validation set:", stats[ks[3]],
-                         stats[ks[4]], stats[ks[5]])
+            if len(stats['user_side_data_column_names']) > 0:
+                ret += "  user side data:\n"
+                colnames = stats['user_side_data_column_names']
+                coltypes = stats['user_side_data_column_types']
+                assert len(colnames) == len(coltypes), \
+                    "Error importing user side data: mismatch between the \
+                     number of column names and column types."
+
+                for i in range(len(colnames)):
+                    ret += '    {0} : {1}\n'.format(colnames[i], coltypes[i])
+
+            if len(stats['item_side_data_column_names']) > 0:
+                ret += "  item side data:\n"
+                colnames = stats['item_side_data_column_names']
+                coltypes = stats['item_side_data_column_types']
+                assert len(colnames) == len(coltypes), \
+                    "Error importing item side data: mismatch between the \
+                     number of column names and column types."
+
+                for i in range(len(colnames)):
+                    ret += '    {0} : {1}\n'.format(colnames[i], coltypes[i])
+
 
             # Training summary
-            ks = ["elapsed_time", "random_seed", "holdout_probability",
+            ks = ["elapsed_time",
+                  "random_seed",
                   "data_load_elapsed_time"]
-            if all([k in stats.keys()]):
+            if any([k in stats.keys()]):
                 ret += "\nTraining summary\n"
+            if 'elapsed_time' in stats.keys():
                 ret += "  %-20s %fs\n" % ("training time:", stats["elapsed_time"])
+            if 'data_load_elapsed_time' in stats.keys():
                 ret += "  %-20s %fs\n" % ("data load time:", stats["data_load_elapsed_time"])
 
-                if "validation_metrics_elapsed_time" in stats.keys():
-                    ret += "  %-20s %fs\n" % \
+            if "validation_metrics_elapsed_time" in stats.keys():
+                ret += "  %-20s %fs\n" % \
                            ("metrics time:", stats["validation_metrics_elapsed_time"])
 
-                # Print holdout probability only if nonzero
-                p = options['holdout_probability']
-                if p > 0:
-                    ret += "  heldout %.3f of data for validation\n" % p
-
-                # Print random seed only if nonzero
-                if options['random_seed'] != 0:
-                    ret += "  random seed: %20d \n" % options['random_seed']
-
             # If available, print performance statistics
             for k in stats.keys():
                 if 'rmse' in k or 'precision' in k or 'recall' in k:
@@ -145,8 +164,10 @@ class RecommenderModel(Model):
             ret += '\nThis model has yet to be trained.\n'
 
         # Remove any options that should not be shown under "Settings"
-        to_ignore = ['random_seed', 'holdout_prob', 'user_column',
-                     'item_column', 'target_column']
+        to_ignore = ['random_seed',
+                     'user_column',
+                     'item_column',
+                     'target_column']
         for k in to_ignore:
             if k in options:
                 del options[k]
@@ -204,7 +225,7 @@ class RecommenderModel(Model):
         response = _graphlab.toolkits.main.run('recsys_set_current_options', opts)
         return response
 
-    def predict(self, dataset):
+    def score(self, dataset):
         """
         Return a score prediction for the user ids and item ids in the provided
         data set.
@@ -216,11 +237,11 @@ class RecommenderModel(Model):
 
         Returns
         -------
-        out : DataFrame/SFrame:
-            A DataFrame with model predictions, with columns user_id, item_id, and
-            score
+        out : SArray
+            An SArray with scores for each given user-item pair predicted by
+            the model.
         """
-        _mt._get_metric_tracker().track('toolkit.recsys.predict')
+        _mt._get_metric_tracker().track('toolkit.recsys.score')
 
         if type(dataset) == _pd.DataFrame:
             dataset = _SFrame(dataset)
@@ -231,56 +252,98 @@ class RecommenderModel(Model):
         # Call the C++ function for recommender_model
         response = _graphlab.toolkits.main.run('recsys_predict', opts)
         result = _SFrame(None, _proxy=response['data'])
-        return result
+        return result['prediction']
 
-    def recommend(self, dataset, users=None, k=30):
+    def recommend(self, users=None, k=10, exclude=None, items=None,
+                  new_observation_data=None, new_user_data=None, new_item_data=None,
+                  exclude_known=True):
         """
-        Return the k best recommend items for each user, excluding items for
-        which the user already has an entry. Typically this is used after the
-        model has been trained. For a RecommenderModel object m, we obtain the
-        highest ranked items for each user in our training set by running:
-
-        >>> m.train(data, 'user', 'item')
-        >>> recs = m.recommend(data)
+        Recommends the k-highest scored items for each user in users.
 
         Parameters
         ----------
-        dataset : SFrame/pandas.DataFrame
-            A dataset of observed data for which recommendations are desired.
-            The k highest ranked items are returned for each user in this
-            dataset, and all observations in this data set will be excluded from
-            the recommendations.
+        users:  SArray or list; optional
+            An :class:`~graphlab.SArray` (or list) of users for which to make recommendations.  If 'None', then recommend(...) generates predictions for all users in the training set.
 
-        users : SArray
-            If provided, recommendations will only be returned for this set of users.
+        k: int, optional
+            The number of recommendations to generate for each user.
 
-        k : int, optional
-            The number of items to return for each user (default is 30).
+        items: SArray, optional
+            To restrict the set of items under consideration, the user may pass in an SArray of items from which the recommended items will be chosen.  This allows one to choose only items that are, for instance, in a particular genre or category.   The default is that all items are under consideration.
+
+        new_observation_data: SFrame, optional
+            ``new_observation_data`` may give additional observation data to the model.  Any overlap with the training data will be ignored.  Must be in the same format as the observation data passed to ``create``.
+
+        new_user_data: SFrame, optional
+            ``new_user_data`` may give additional user data to the model.  Any overlap with the training data will be ignored.  Must be in the same format as the user data passed to ``create``.
+
+        new_item_data: SFrame, optional
+            ``new_item_data`` may give additional item data to the model.  Any overlap with the training data will be ignored.  Must be in the same format as the item data passed to ``create``.
+
+        exclude: SFrame, optional
+            An :class:`~graphlab.SFrame` of user / item pairs.  The column names must be equal to the user and item columns of the main data, and it provides the model with user/item pairs to exclude from the recommendations.  These user-item-pairs are always excluded from the predictions, even if exclude_known is False.
+
+        exclude_known: bool
+            By default, all user-item interactions previously seen in the training data, or in any new data provided using new_observation_data=..., are excluded from the recommendations.  Passing in ``exclude_known = False`` overrides this behavior.
 
         Returns
         -------
         out : SFrame
-            A SFrame with the top ranked items for each user. The
-            columns are: *user*, *item*, *score*, and *rank*.
+            A SFrame with the top ranked items for each user. The columns are: ``user_column``, ``item_column``, *score*, and *rank*, where ``user_column`` and ``item_column`` match the user and item column names specified at training time.  The rank column is between 1 and ``k`` and gives the relative score of that item.  The value of score depends on the method used for recommendations.
+
+
         """
         _mt._get_metric_tracker().track('toolkit.recsys.recommend')
+        assert type(k) == int
 
-        if type(dataset) == _pd.DataFrame:
-            dataset = _SFrame(dataset)
-        elif type(dataset) != _SFrame:
-            raise NotImplemented
-
-        # TEMP: Get unique list of users on Python side
         if users is None:
-            users = list(set(dataset[self.get('user_column')]))
-        users_sf = _SFrame()
-        users_sf['users'] = users
+            users = _SArray()
+        if exclude is None:
+            exclude = _SFrame()
+        if items is None:
+            items = _SArray()
+        if new_observation_data is None:
+            new_observation_data = _SFrame()
+        if new_user_data is None:
+            new_user_data = _SFrame()
+        if new_item_data is None:
+            new_item_data = _SFrame()
 
-        opt = {}
-        opt["model"] = self.__proxy__
-        opt["users"] = users_sf
-        opt["top_k"] = k
-        opt['available_data'] = dataset
+        if isinstance(users, list) or isinstance(users, _np.ndarray):
+            users = _SArray(users)
+        if isinstance(exclude, _pd.DataFrame) or isinstance(exclude, _np.ndarray):
+            exclude = _SFrame(exclude)
+        if isinstance(items, list) or isinstance(items, _np.ndarray):
+            items = _SArray(items)
+        if isinstance(new_observation_data, _pd.DataFrame) or isinstance(new_observation_data, _np.ndarray):
+            new_observation_data = _SFrame(new_observation_data)
+        if isinstance(new_user_data, _pd.DataFrame) or isinstance(new_user_data, _np.ndarray):
+            new_user_data = _SFrame(new_user_data)
+        if isinstance(new_item_data, _pd.DataFrame) or isinstance(new_item_data, _np.ndarray):
+            new_item_data = _SFrame(new_item_data)
+
+        def check_type(arg, arg_name, required_type, allowed_types):
+            if not isinstance(arg, required_type):
+                raise TypeError("Parameter " + arg_name + " must be of type(s) "
+                                + (", ".join(allowed_types) )
+                                + "; Type '" + str(type(arg)) + "' not recognized.")
+
+        check_type(users, "users", _SArray, ["SArray", "list", "numpy.ndarray"])
+        check_type(exclude, "exclude", _SFrame, ["SFrame"])
+        check_type(items, "items", _SArray, ["SArray", "list", "numpy.ndarray"])
+        check_type(new_observation_data, "new_observation_data", _SFrame, ["SFrame"])
+        check_type(new_user_data, "new_user_data", _SFrame, ["SFrame"])
+        check_type(new_item_data, "new_item_data", _SFrame, ["SFrame"])
+
+        opt = {'model': self.__proxy__,
+               'users': users,
+               'top_k': k,
+               'exclude': exclude,
+               'items': items,
+               'new_data': new_observation_data,
+               'new_user_data': new_user_data,
+               'new_item_data': new_item_data,
+               'exclude_known': exclude_known*1}
         response = _graphlab.toolkits.main.run('recsys_recommend', opt)
         recs = _SFrame(None, _proxy=response['data'])
         return recs
@@ -290,98 +353,51 @@ class RecommenderModel(Model):
         Get information about model creation, e.g. time elapsed during
         model fitting, data loading, and more.
 
+        Note: This method will be *deprecated* soon. Please use m.summary()
+        instead.
+
         Returns
         -------
         out : dict
             Statistics about model training, e.g. runtime.
+
         """
+        _logging.warning("This method will be deprecated soon. Please use m.summary().")
         _mt._get_metric_tracker().track('toolkit.recsys.training_stats')
 
         opts = {'model': self.__proxy__}
         response = _graphlab.toolkits.main.run("recsys_get_stats", opts)
         return response
 
-    def get_validation_split(self, dataset):
-        """
-        Retrieve the subset of the training dataset that was used internally for
-        validation testing.
-
-        Parameters
-        ----------
-        dataset : DataFrame/SFrame
-            A dataset in the same format as the one used during training.
-
-        Returns
-        -------
-        out : dict
-            A dictionary with two items. The *training_data* item contains the
-            data used for training, and the *validation_data* item contains the
-            data used for internal validation.
-        """
-
-        _mt._get_metric_tracker().track('toolkit.recsys.get_validation_split')
-
-        if type(dataset) == _pd.DataFrame:
-            dataset = _SFrame(dataset)
-        elif type(dataset) != _SFrame:
-            raise NotImplementedError
-
-        opts = {'model': self.__proxy__, 'data': dataset}
-        response = _graphlab.toolkits.main.run('recsys_get_validation_set', opts)
-        ret = {'training_data': _SFrame(None, _proxy=response['training_data']),
-               'validation_data': _SFrame(None, _proxy=response['validation_data'])}
-        return ret
-
-    def _evaluate_precision_recall(self, dataset, cutoffs=[5, 10, 15],
-                                   skip_set=None):
+    def _evaluate_precision_recall(self, dataset, cutoffs=[5, 10, 15], skip_set=None):
         """
         Compute a model's precision and recall for a particular dataset.
 
         Parameters
         ----------
-        dataset : pandas.DataFrame/SFrame
-            A DataFrame in the same format as the one used during training.
+        dataset : SFrame
+            An SFrame in the same format as the one used during training.
+            This will be compared to the model's recommendations, which exclude
+            the (user, item) pairs seen at training time.
 
         cutoffs : list
             A list of cutoff values for which one wants to evaluate precision
             and recall, i.e. the value of k in precision@k.
 
-        skip_set : SFrame/DataFrame
-            A dataset in the same format as the one used for training. Each (user,
-            item) pair in the skip_set will be excluded from the set of
-            recommendations.
-
         Returns
         -------
-        out : dict
-            A dictionary containing two items, both SFrames. Item *ranked_items*
-            contains the recommendations for each user, and item *results* contains
-            the precision and recall at each cutoff value.
+        out : SFrame
+            Contains the precision and recall at each cutoff value and each
+            user in ``dataset``.
         """
 
-        if type(dataset) == _pd.DataFrame:
-            dataset = _SFrame(dataset)
-        elif type(dataset) != _SFrame:
-            raise NotImplementedError
-        if skip_set is None:
-            skip_set = _SFrame()
-            for colname in dataset.column_names():
-                skip_set[colname] = _SArray()
-        if type(skip_set) == _pd.DataFrame:
-            skip_set = _SFrame(skip_set)
-        elif type(skip_set) != _SFrame:
-            raise NotImplementedError
+        users = dataset[self.get('user_column')].unique()
 
-        opts = {}
-        opts['model'] = self.__proxy__
-        opts['validation_data'] = dataset
-        opts['available_data'] = skip_set
-        opts['cutoffs'] = _pd.DataFrame({'cutoff': cutoffs})
-        opts['top_k'] = max(cutoffs)
-        response = _graphlab.toolkits.main.run("recsys_precision_recall", opts)
-        res = {'ranked_items': _SFrame(None, _proxy=response['ranked_items']),
-               'results': _SFrame(None, _proxy=response['results'])}
-        return res
+        dataset = dataset[[self.get('user_column'), self.get('item_column')]]
+        
+        recs = self.recommend(users=users, k=max(cutoffs), exclude=skip_set)
+        pr = _graphlab.recommender.util.precision_recall_by_user(dataset, recs, cutoffs)
+        return pr
 
     def _evaluate_rmse(self, dataset):
         """
@@ -403,19 +419,38 @@ class RecommenderModel(Model):
         if type(dataset) == _pd.DataFrame:
             dataset = _SFrame(dataset)
 
-        opts = {}
-        opts['model'] = self.__proxy__
-        opts['dataset'] = dataset
-        response = _graphlab.toolkits.main.run('recsys_rmse', opts)
-        rmse_by_user = _SFrame(None, _proxy=response['rmse_by_user'])
-        rmse_by_item = _SFrame(None, _proxy=response['rmse_by_item'])
-        rmse_overall = response['rmse_overall']
+        assert 'target_column' in self.list_fields(), \
+               'RMSE evaluation only valid for models trained using a target.'
+        target_column = self.get('target_column')
+        assert target_column in dataset.column_names(), \
+               'Provided dataset must contain a target column with the same \
+                name as the target used during training.'
+        y = dataset[target_column]
+        yhat = self.score(dataset)
+        user_column = self.get('user_column')
+        item_column = self.get('item_column')
+        assert user_column in dataset.column_names() and \
+               item_column in dataset.column_names(), \
+            'Provided data set must have a column pertaining to user ids and \
+             item ids, similar to what we had during training.'
+
+        result = dataset[[user_column, item_column]]
+        result['sq_error'] = (y - yhat).apply(lambda x: x**2)
+        rmse_by_user = result.groupby(user_column,
+                {'rmse':_graphlab.aggregate.AVG('sq_error'),
+                 'count':_graphlab.aggregate.COUNT})
+        rmse_by_user['rmse'] = rmse_by_user['rmse'].apply(lambda x: x**.5)
+        rmse_by_item = result.groupby(item_column,
+                {'rmse':_graphlab.aggregate.AVG('sq_error'),
+                 'count':_graphlab.aggregate.COUNT})
+        rmse_by_item['rmse'] = rmse_by_item['rmse'].apply(lambda x: x**.5)
+        overall_rmse = result['sq_error'].mean() ** .5
 
         return {'rmse_by_user': rmse_by_user,
                 'rmse_by_item': rmse_by_item,
-                'rmse_overall': rmse_overall}
+                'rmse_overall': overall_rmse}
 
-    def evaluate(self, dataset, verbose=True, **kwargs):
+    def evaluate(self, dataset, verbose=False, **kwargs):
         r"""Evaluate the model's ability to make predictions for another
         dataset. If the model is trained to predict a particular target, the
         default metric used for model comparison is root-mean-squared error
@@ -454,18 +489,21 @@ class RecommenderModel(Model):
             *rmse_by_user* and *rmse_by_item* are SFrames with per-user and
             per-item RMSE, while *rmse_overall* is the overall RMSE (a float).
             If the model is trained without a target (i.e. precision and recall
-            are the evaluation criteria) and SFRame is returned with both of
-            these metrics for each user at several cutoff values.
+            are the evaluation criteria) and :py:class:`~graphlab.SFrame` is
+            returned with both of these metrics for each user at several cutoff
+            values.
         """
 
         _mt._get_metric_tracker().track('toolkit.recsys.evaluate')
 
         # If the model does not have a target column, compute prec-recall.
         if self.get('target_column') is None or self.get('target_column') == '':
-            results = self._evaluate_precision_recall(dataset, **kwargs)['results']
+            results = self._evaluate_precision_recall(dataset, **kwargs)
             if verbose:
                 print "\nPrecision and recall summary statistics by cutoff"
-                print results.to_dataframe().groupby('cutoff').describe()[['precision', 'recall']]
+                print results.groupby('cutoff',
+                        {'mean_precision':_graphlab.aggregate.AVG('precision'),
+                         'mean_recall':_graphlab.aggregate.AVG('recall')}).topk('cutoff', reverse=True)
         else:
             results = self._evaluate_rmse(dataset)
             if verbose:
@@ -480,3 +518,4 @@ class RecommenderModel(Model):
                 print results['rmse_by_item'].topk('rmse', 1)
 
         return results
+
