@@ -38,6 +38,7 @@ def make_internal_url(url):
       Returns the s3 URL with credentials filled in using graphlab.aws.get_aws_credential().
       For example: "s3://mybucket/foo" -> "s3://$AWS_ACCESS_KEY_ID:$AWS_SECRET_ACCESS_KEY:mybucket/foo".
     - URL to other remote locations, e.g. http://, will remain as is.
+    - Expands ~ to $HOME
 
     Parameters
     ----------
@@ -89,6 +90,8 @@ def make_internal_url(url):
         else:
             raise ValueError('Invalid url protocol %s. Supported url protocols are: remote://, local://, s3://, https:// and hdfs://' % protocol)
     elif len(urlsplit) == 1:
+        # expand ~ to $HOME
+        url = os.path.expanduser(url) 
         # url for files on local client, check if we are connecting to local server
         if (isinstance(glconnect.get_server(), server.LocalServer)):
             path_on_server = url
